/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Gemm.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_META_GEMM_H
#define TVMET_META_GEMM_H

namespace tvmet {

namespace meta {

/**
 * \class gemm Gemm.h "tvmet/meta/Gemm.h"
 * \brief Meta class for matrix-matrix operations, like product.
 * \note The rows of matrix 2 have to be equal to cols of matrix 1, square
 *       matrix is mandatory.
 */
template<std::size_t Rows1, std::size_t Cols1,
	 std::size_t Cols2,
	 std::size_t RowStride1, std::size_t ColStride1,
	 std::size_t RowStride2, std::size_t ColStride2,
	 std::size_t K>
class gemm
{
private:
  gemm();
  gemm& operator=(const gemm<Rows1, Cols1, Cols2, RowStride1, ColStride1, RowStride2, ColStride2, K>&);

private:
  enum {
    doIt = (K != Cols1 - 1) 		/**< recursive counter */
  };

public:
  template<class T1, class T2>
  static inline
  typename PromoteTraits<T1, T2>::value_type
  product(const T1* _tvmet_restrict lhs, const T2* _tvmet_restrict rhs, std::size_t i, std::size_t j) {
    return lhs[i * RowStride1 + K * ColStride1] * rhs[K * RowStride2 + j * ColStride2]
      + gemm<Rows1 * doIt, Cols1 * doIt,
               Cols2 * doIt, RowStride1 * doIt, ColStride1 * doIt,
               RowStride2 * doIt, ColStride2 * doIt, (K+1) * doIt>::product(lhs, rhs, i, j);
  }
};

/**
 * \class gemm<0,0,0,0,0,0,0,0> Gemm.h "tvmet/meta/Gemm.h"
 * \brief gemm Specialized for recursion.
 */
template<>
class gemm<0,0,0,0,0,0,0,0> {
  gemm();
  gemm& operator=(const gemm<0,0,0,0,0,0,0,0>&);

public:
  static inline
  XprNull product(const void*, const void*, std::size_t, std::size_t) {
    return XprNull();
  }
};

} // namespace meta

} // namespace tvmet

#endif /* TVMET_META_GEMM_H */

// Local Variables:
// mode:C++
// End:
