/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Gemv.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_META_GEMV_H
#define TVMET_META_GEMV_H

namespace tvmet {

namespace meta {

/**
 * \class gemv Gemv.h "tvmet/meta/Gemv.h"
 * \brief Meta class for matrix-vector operations
 */
template<std::size_t Rows, std::size_t Cols, std::size_t RowStride, std::size_t ColStride,
	 std::size_t VecStride, std::size_t J>
class gemv
{
  gemv();
  gemv& operator=(const gemv<Rows, Cols, RowStride, ColStride, VecStride, J>&);

private:
  enum {
    doIt = J < (Cols-1)  		/**< recursive counter */
  };

public:
  template<class T1, class T2>
  static inline
  typename PromoteTraits<T1, T2>::value_type
  product(const T1* _tvmet_restrict m, const T2* _tvmet_restrict v, std::size_t i) {
    return m[i * RowStride + J * ColStride] * v[J * VecStride]
      + gemv<Rows * doIt, Cols * doIt,
               RowStride * doIt, ColStride * doIt,
               VecStride * doIt, (J+1)*doIt>::product(m, v, i);
  }
};

/**
 * \class gemv<0,0,0,0,0,0> Gemv.h "tvmet/meta/Gemv.h"
 * \brief gemv Specialized for recursion
 */
template<>
class gemv<0,0,0,0,0,0>
{
  gemv();
  gemv& operator=(const gemv<0,0,0,0,0,0>&);

public:
  static inline
  XprNull product(const void*, const void*, std::size_t) {
    return XprNull();
  }
};

} // namespace meta

} // namespace tvmet

#endif /* TVMET_META_GEMV_H */

// Local Variables:
// mode:C++
// End:
