/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ConstRef.h,v 1.1 2003/03/20 19:47:24 opetzold Exp $
 */

#ifndef TVMET_XPR_CONSTREF_H
#define TVMET_XPR_CONSTREF_H

namespace tvmet {

#if 0 // unused

/**
 * \class XprConstReference Xpr.h "tvmet/Xpr.h"
 * \brief Wrapper class for avoiding temporary objects used by operator
 * expressions.
 * \internal UNUSED
 */
template<class E>
class XprConstReference : public TvmetBase< XprConstReference<E> >
{
  XprConstReference();
  XprConstReference& operator=(const XprConstReference&);

 public:
  typedef E						expr_type;
  typedef typename expr_type::value_type		value_type;

 public:
  /** Constructor. */
  explicit XprConstReference(const expr_type& e) : m_expr(e) { }

  /** Copy Constructor. Not explicit! */
  XprConstReference(const XprConstReference<expr_type>& rhs) : m_expr(rhs.m_expr) { }

  /** index operator, returns the expression by index. */
  value_type operator[](std::size_t i) const { return m_expr[i]; }

  /** index operator for arrays/matrices */
  value_type operator()(std::size_t i, std::size_t j) const { return m_expr(i, j); }

public: // debugging Xpr parse tree
  void print_on(std::ostream& os, std::size_t l=0) const {
    os << IndentLevel(l++) << "XprConstReference<" << std::endl;
    m_expr.print_on(os, l);
    os << IndentLevel(l) << ">," << std::endl;
  }

private:
  const E& _tvmet_restrict 				m_expr;
};

#endif

} // namespace tvmet

#endif // TVMET_XPR_CONSTREF_H

// Local Variables:
// mode:C++
// End:
