/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Eval.h,v 1.1 2003/03/20 19:47:24 opetzold Exp $
 */

#ifndef TVMET_XPR_EVAL_H
#define TVMET_XPR_EVAL_H

namespace tvmet {

/**
 * \class XprEval Eval.h "tvmet/xpr/Eval.h"
 * \brief evaluate the expression
 *
 * Since we can't overwrite the ? operator we have to write a wrapper
 * for expression like return v1>v2 ? true : false
 */
template<class E1, class E2, class E3>
class XprEval : public TvmetBase< XprEval<E1, E2, E3> >
{
public:
  typedef E1 						expr1_type;
  typedef E2 						expr2_type;
  typedef E3 						expr3_type;

  typedef typename expr2_type::value_type 		value2_type;
  typedef typename expr3_type::value_type 		value3_type;

  typedef typename
  PromoteTraits<value2_type, value3_type>::value_type 	value_type;

private:
  XprEval();
  XprEval& operator=(const XprEval<expr1_type, expr2_type, expr3_type>&);

public:
  /** Constructor. */
#ifdef TVMET_OPTIMIZE_XPR_CTOR_ARGS_BY_VALUE
  explicit XprEval(expr1_type expr1, expr2_type expr2, expr3_type expr3)
    : m_expr1(expr1), m_expr2(expr2), m_expr3(expr3) { }
#else
  explicit XprEval(const expr1_type& expr1, const expr2_type& expr2, const expr3_type& expr3)
    : m_expr1(expr1), m_expr2(expr2), m_expr3(expr3) { }
#endif

  /** Copy Constructor. Not explicit! */
#ifdef TVMET_OPTIMIZE_XPR_MANUAL_CCTOR
  XprEval(const XprEval<expr1_type, expr2_type, expr3_type>& expr)
    : m_expr1(expr.m_expr1), m_expr2(expr.m_expr2), m_expr3(expr.m_expr3) { }
#endif

public: //access
  /** index operator for vectors. */
  value_type operator[](std::size_t i) const {
    return m_expr1[i] ? m_expr2[i] : m_expr3[i];
  }

  /** index operator for matrizes. */
  value_type operator()(std::size_t i, std::size_t j) const {
    return m_expr1(i, j) ? m_expr2(i, j) : m_expr3(i, j);
  }

public: // debugging Xpr parse tree
  void print_on(std::ostream& os, std::size_t l=0) const {
    os << IndentLevel(l++) << "XprEval<" << std::endl;
    m_expr1.print_on(os, l);
    m_expr2.print_on(os, l);
    m_expr3.print_on(os, l);
    os << IndentLevel(--l) << ">," << std::endl;
  }

private:
  expr1_type 						m_expr1;
  expr2_type 						m_expr2;
  expr3_type 						m_expr3;
};

} // namespace tvmet

#endif // TVMET_XPR_EVAL_H

// Local Variables:
// mode:C++
// End:
