/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MVProduct.h,v 1.2 2003/03/27 16:40:59 opetzold Exp $
 */

#ifndef TVMET_XPR_MVPRODUCT_H
#define TVMET_XPR_MVPRODUCT_H

namespace tvmet {

/* forwards */
namespace meta {

template<std::size_t Rows, std::size_t Cols, std::size_t RowStride, std::size_t ColStride,
	 std::size_t VecStride, std::size_t J>
class gemv;

} // namespace meta

/**
 * \class XprMVProduct MVProduct.h "tvmet/xpr/MVProduct.h"
 * \brief Expression for matrix-vector product
 */
template<class T1, std::size_t Rows, std::size_t Cols,
	 std::size_t RowStride, std::size_t ColStride,
	 class T2, std::size_t VecStride>
class XprMVProduct
  : public TvmetBase< XprMVProduct<T1, Rows, Cols, RowStride, ColStride, T2, VecStride> >
{
  XprMVProduct();
  XprMVProduct& operator=(const XprMVProduct&);

public:
  typedef typename PromoteTraits<T1, T2>::value_type	value_type;

public:
  /** Constructor. */
  explicit XprMVProduct(const T1* mtx, const T2* vec)
    : m_mtx(mtx), m_vec(vec) { }

  /** Copy Constructor. Not explicit! */
#ifdef TVMET_OPTIMIZE_XPR_MANUAL_CCTOR
  XprMVProduct(const XprMVProduct<T1, Rows, Cols, RowStride, ColStride, T2, VecStride>& e)
    : m_mtx(e.m_mtx), m_vec(e.m_vec) { }
#endif

  /** index operator, returns the expression by index. This is the vector
      style since a matrix*vector gives a vector. */
  value_type operator[](std::size_t i) const {
    return meta::gemv<Rows, Cols, RowStride,
                      ColStride, VecStride, 0>::product(m_mtx, m_vec, i);
  }

public: // debugging Xpr parse tree
  void print_on(std::ostream& os, std::size_t l=0) const {
    os << IndentLevel(l++) << "XprMVProduct<" << std::endl;
    os << IndentLevel(l) << typeid(T1).name() << ", " << Rows << ", " << Cols << ", ";
    os << RowStride << ", " << ColStride << ", ";
    os << typeid(T2).name() << ", " << VecStride << std::endl;
    os << IndentLevel(--l) << ">," << std::endl;
  }

private:
  const T1* _tvmet_restrict 				m_mtx;
  const T2* _tvmet_restrict 				m_vec;
};

} // namespace tvmet

#endif // TVMET_XPR_MVPRODUCT_H

// Local Variables:
// mode:C++
// End:
